@@ -4,17 +4,14 @@ use unicode_segmentation::UnicodeSegmentation;
 
 use crate::ansi;
 use crate::cli;
-use crate::config::Config;
+use crate::config::{self, delta_unreachable, Config};
 use crate::delta::State;
-use crate::features::line_numbers;
-use crate::features::OptionValueFunction;
+use crate::edits;
+use crate::features::{line_numbers, OptionValueFunction};
 use crate::minusplus::*;
-use crate::paint::Painter;
-use crate::paint::{BgFillMethod, BgShouldFill};
+use crate::paint::{BgFillMethod, BgShouldFill, LineSections, Painter};
 use crate::style::Style;
-use crate::wrapping::wrap_zero_block;
-
-pub type LineSections<'a, S> = Vec<(S, &'a str)>;
+use crate::wrapping::{wrap_minusplus_block, wrap_zero_block};
 
 pub fn make_feature() -> Vec<(String, OptionValueFunction)> {
     builtin_feature!([
@@ -36,6 +33,8 @@ pub use crate::minusplus::MinusPlusIndex as PanelSide;
 pub use MinusPlusIndex::Minus as Left;
 pub use MinusPlusIndex::Plus as Right;
 
+use super::line_numbers::LineNumbersData;
+
 #[derive(Debug)]
 pub struct Panel {
     pub width: usize,
@@ -94,7 +93,7 @@ pub fn has_long_lines(
     let mut wrapping_lines = LeftRight::default();
 
     let mut check_if_too_long = |side| {
-        let lines_side: &Vec<(String, State)> = lines[side];
+        let lines_side: &[(String, State)] = lines[side];
         wrapping_lines[side] = lines_side
             .iter()
             .map(|(line, _)| line_is_too_long(line, line_width[side]))
@@ -108,41 +107,104 @@ pub fn has_long_lines(
     (wrap_any[Left] || wrap_any[Right], wrapping_lines)
 }
 
-/// Emit a sequence of minus and plus lines in side-by-side mode.
 #[allow(clippy::too_many_arguments)]
-pub fn paint_minus_and_plus_lines_side_by_side<'a>(
-    syntax_left_right: LeftRight<Vec<LineSections<'a, SyntectStyle>>>,
-    diff_left_right: LeftRight<Vec<LineSections<'a, Style>>>,
-    states_left_right: LeftRight<Vec<State>>,
+pub fn paint_minus_and_plus_lines_side_by_side(
+    lines: LeftRight<&Vec<(String, State)>>,
+    syntax_sections: LeftRight<Vec<LineSections<SyntectStyle>>>,
+    diff_sections: LeftRight<Vec<LineSections<Style>>>,
+    lines_have_homolog: LeftRight<Vec<bool>>,
     line_alignment: Vec<(Option<usize>, Option<usize>)>,
+    line_numbers_data: &mut Option<LineNumbersData>,
     output_buffer: &mut String,
-    config: &Config,
-    line_numbers_data: &mut Option<&mut line_numbers::LineNumbersData>,
-    background_color_extends_to_terminal_width: LeftRight<BgShouldFill>,
+    config: &config::Config,
 ) {
+    let line_states = LeftRight::new(
+        lines[Left].iter().map(|(_, state)| state.clone()).collect(),
+        lines[Right]
+            .iter()
+            .map(|(_, state)| state.clone())
+            .collect(),
+    );
+
+    let line_numbers_data = line_numbers_data
+        .as_mut()
+        .unwrap_or_else(|| delta_unreachable("side-by-side requires Some(line_numbers_data)"));
+
+    let bg_should_fill = LeftRight::new(
+        // Using an ANSI sequence to fill the left panel would not work.
+        BgShouldFill::With(BgFillMethod::Spaces),
+        // Use what is configured for the right side.
+        BgShouldFill::With(config.line_fill_method),
+    );
+
+    // Only set `should_wrap` to true if wrapping is wanted and lines which are
+    // too long are found.
+    // If so, remember the calculated line width and which of the lines are too
+    // long for later re-use.
+    let (should_wrap, line_width, long_lines) = {
+        if config.wrap_config.max_lines == 1 {
+            (false, LeftRight::default(), LeftRight::default())
+        } else {
+            let line_width = available_line_width(config, line_numbers_data);
+
+            let (should_wrap, long_lines) = has_long_lines(&lines, &line_width);
+
+            (should_wrap, line_width, long_lines)
+        }
+    };
+
+    let (line_alignment, line_states, syntax_sections, diff_sections) = if should_wrap {
+        // Calculated for syntect::highlighting::style::Style and delta::Style
+        wrap_minusplus_block(
+            config,
+            syntax_sections,
+            diff_sections,
+            &line_alignment,
+            &line_width,
+            &long_lines,
+        )
+    } else {
+        (line_alignment, line_states, syntax_sections, diff_sections)
+    };
+    let lines_have_homolog = if should_wrap {
+        edits::make_lines_have_homolog(&line_alignment)
+    } else {
+        lines_have_homolog
+    };
+
     for (minus_line_index, plus_line_index) in line_alignment {
         output_buffer.push_str(&paint_left_panel_minus_line(
             minus_line_index,
-            &syntax_left_right[Left],
-            &diff_left_right[Left],
+            &syntax_sections[Left],
+            &diff_sections[Left],
+            &lines_have_homolog[Left],
             match minus_line_index {
-                Some(i) => &states_left_right[Left][i],
+                Some(i) => &line_states[Left][i],
                 None => &State::HunkMinus(None),
             },
-            line_numbers_data,
-            background_color_extends_to_terminal_width[Left],
+            &mut Some(line_numbers_data),
+            bg_should_fill[Left],
             config,
         ));
+
+        // HACK: The left line number is not getting incremented in `linenumbers_and_styles()`
+        // when the alignment matches a minus with a plus line, so fix that here.
+        // This information should be passed down into `paint_line()` to set `increment` to true.
+        if minus_line_index.is_some() && plus_line_index.is_some() {
+            line_numbers_data.line_number[Left] += 1;
+        }
+
         output_buffer.push_str(&paint_right_panel_plus_line(
             plus_line_index,
-            &syntax_left_right[Right],
-            &diff_left_right[Right],
+            &syntax_sections[Right],
+            &diff_sections[Right],
+            &lines_have_homolog[Right],
             match plus_line_index {
-                Some(i) => &states_left_right[Right][i],
+                Some(i) => &line_states[Right][i],
                 None => &State::HunkPlus(None),
             },
-            line_numbers_data,
-            background_color_extends_to_terminal_width[Right],
+            &mut Some(line_numbers_data),
+            bg_should_fill[Right],
             config,
         ));
         output_buffer.push('\n');
@@ -192,6 +254,7 @@ pub fn paint_zero_lines_side_by_side<'a>(
                 panel_line_is_empty,
                 Some(line_index),
                 &diff_style_sections,
+                None,
                 &state,
                 *panel_side,
                 background_color_extends_to_terminal_width,
@@ -208,6 +271,7 @@ fn paint_left_panel_minus_line<'a>(
     line_index: Option<usize>,
     syntax_style_sections: &[LineSections<'a, SyntectStyle>],
     diff_style_sections: &[LineSections<'a, Style>],
+    lines_have_homolog: &[bool],
     state: &'a State,
     line_numbers_data: &mut Option<&mut line_numbers::LineNumbersData>,
     background_color_extends_to_terminal_width: BgShouldFill,
@@ -227,6 +291,7 @@ fn paint_left_panel_minus_line<'a>(
         panel_line_is_empty,
         line_index,
         diff_style_sections,
+        Some(lines_have_homolog),
         state,
         Left,
         background_color_extends_to_terminal_width,
@@ -241,6 +306,7 @@ fn paint_right_panel_plus_line<'a>(
     line_index: Option<usize>,
     syntax_style_sections: &[LineSections<'a, SyntectStyle>],
     diff_style_sections: &[LineSections<'a, Style>],
+    lines_have_homolog: &[bool],
     state: &'a State,
     line_numbers_data: &mut Option<&mut line_numbers::LineNumbersData>,
     background_color_extends_to_terminal_width: BgShouldFill,
@@ -261,6 +327,7 @@ fn paint_right_panel_plus_line<'a>(
         panel_line_is_empty,
         line_index,
         diff_style_sections,
+        Some(lines_have_homolog),
         state,
         Right,
         background_color_extends_to_terminal_width,
@@ -270,10 +337,12 @@ fn paint_right_panel_plus_line<'a>(
     panel_line
 }
 
+#[allow(clippy::too_many_arguments)]
 fn get_right_fill_style_for_panel<'a>(
     line_is_empty: bool,
     line_index: Option<usize>,
     diff_style_sections: &[LineSections<'a, Style>],
+    lines_have_homolog: Option<&[bool]>,
     state: &State,
     panel_side: PanelSide,
     background_color_extends_to_terminal_width: BgShouldFill,
@@ -293,6 +362,7 @@ fn get_right_fill_style_for_panel<'a>(
             let (bg_fill_mode, fill_style) =
                 Painter::get_should_right_fill_background_color_and_fill_style(
                     &diff_style_sections[index],
+                    lines_have_homolog.map(|h| h[index]),
                     state,
                     background_color_extends_to_terminal_width,
                     config,
@@ -389,6 +459,7 @@ fn pad_panel_line_to_width<'a>(
     panel_line_is_empty: bool,
     line_index: Option<usize>,
     diff_style_sections: &[LineSections<'a, Style>],
+    lines_have_homolog: Option<&[bool]>,
     state: &State,
     panel_side: PanelSide,
     background_color_extends_to_terminal_width: BgShouldFill,
@@ -426,6 +497,7 @@ fn pad_panel_line_to_width<'a>(
         panel_line_is_empty,
         line_index,
         diff_style_sections,
+        lines_have_homolog,
         state,
         panel_side,
         background_color_extends_to_terminal_width,
